// BaseDialog.cpp: implementation of the CBaseDialog class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BaseDialog.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HWND CBaseDialog::m_hWindow = NULL;
long CBaseDialog::m_lSaveThis = 0;

CBaseDialog::CBaseDialog(int nResId, HWND hParent)
{
	m_lSaveThis = (long)this; /// store this pointer

	m_nResId = nResId;
	m_hParent = hParent;

}

CBaseDialog::~CBaseDialog()
{
	m_hWindow = NULL;
	m_lSaveThis = 0;
}

int CBaseDialog::DoModal(void)
{
	return(DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(m_nResId), m_hParent, DialogProcStatic));
}

void CBaseDialog::AddMessageHandler(UINT MessageId, void(CBaseDialog::*MsgHandler)(HWND hDlg,WPARAM wParam,LPARAM lParam))
{
	t_MessageEntry MessageEntry;
	MessageEntry.MsgHandler = MsgHandler;

	m_MessageMap.insert(std::map<UINT,t_MessageEntry>::value_type(MessageId, MessageEntry)); /// insert key & data to map
}

BOOL CALLBACK CBaseDialog::DialogProcStatic(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if(m_hWindow == NULL)
	{
		m_hWindow = hDlg;
	}

	CBaseDialog *pThis = (CBaseDialog*)m_lSaveThis; /// typecast stored this-pointer to CBaseDialog pointer

	return(pThis->DialogProc(hDlg, message, wParam, lParam));
}

BOOL CALLBACK CBaseDialog::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	m_MessageMapIterator = m_MessageMap.find(message); /// find message entry by key

	if(m_MessageMapIterator == m_MessageMap.end()) /// check if message entry available
	{
		return FALSE;
	}
	else
	{
		t_MessageEntry MessageEntry = (*m_MessageMapIterator).second; /// dereference iterator and get message entry

		void (CBaseDialog::*MessageHandler)(HWND hDlg,WPARAM wParam,LPARAM lParam);
				
		MessageHandler = MessageEntry.MsgHandler;

		(this->*MessageHandler)(hDlg, wParam, lParam); /// execute function

		return TRUE;
	}
}

void CBaseDialog::OnOK(void)
{
	EndDialog(m_hWindow, IDOK);
}

void CBaseDialog::OnCancel(void)
{
	EndDialog(m_hWindow, IDCANCEL);
}

